/*
 * autore Paolo
 * 02/02/2014
 *
 * Lo scopo  quello di generare sei segnali PWM per il pilotaggio di servocomandi
 * analogici Futaba 3003 o similari
 * Il periodo del segnale  di 20 ms con un un impulso minimo di 0.5ms. L'ampiezza
 * massima dell'impulso  uguale alla minima pi quella definita dal potenziometro.
 * Il potenziometro definisce un tempo compreso tra 0 e 2ms.
 * L'impulso generato varia quindi da 0,5mS a 2,5ms.
 * Il valore dei potenziometri po essere qualsiesi meglio se alto (es 47K) per non
 *  caricare l'alimentazione.
 * Controllare che il potenziometro vada a fondo scala (0 ohm) da ambedue i lati.
 * Il potenziometro deve essere lineare, per avere uno spostamento lineare.
 * La posizione di riposo del Futaba 3003 1,5mS di impulso corrisponde a met della
 *  corsa del potenziometro.
 *  Il processore impiegato  un MSP430G2553
 * I pwm generati sono tre, uno con il timer A gli altri due con A1
 * I potenziometri sono collegati con le estremit alla stessa alimentazione del micro
 * e con i cursori alle porte P1.3 P1.4 e P1.5
 * Le uscite dei PWM sono P1.2, P2.1 e P2.4
 * Il refreshing dei dati e definita dal WDT
 */


#include <msp430.h> 

#define periodo       20000			// pari a 20 ms
#define min_duty		500			// pari a 0,5 ms


unsigned int adc_val[3];    // Contiene i dati letti da ADC


/*
 * main.c
 */
int main(void) {
	WDTCTL = WDT_MDLY_32;            // WDT ~45ms intervallo di tempo
	IE1 |= WDTIE;                    // abilita interrupt di WDT

	 // Set del clock
	if (CALBC1_1MHZ==0xFF){			// mancando le costanti di calibrazione
		while(1);                   // non proseguire
	}
	DCOCTL = 0;                   	//  DCOx  = 0 e MODx = 0
	BCSCTL1 = CALBC1_1MHZ;          // frequenza del DCO 1MHz
	DCOCTL = CALDCO_1MHZ;


	// Set delle porte
	P1DIR |= 0x04;          // P1.2 = Uscita del segnale
	P1SEL |= 0x04;          // P1.2 = Uscita per la periferica TimerA
	P2DIR |= 0x12;			// P2.1 P2.4 uscite del segnale
	P2SEL |= 0x12;			// P2.1 P2.4 uscite per la periferica TimerA1


	//Impostazioni del convertitore analogico digitale
	ADC10CTL1 = INCH_5+CONSEQ_1;			// porta pi alta, in sequenza
	ADC10CTL0 = ADC10SHT_2 +MSC+ ADC10ON;	// Sample e Hold=16 cicli, adc10 ON
	ADC10DTC1 = 0x03;                       // 3 conversioni in sequenza
	ADC10AE0 |= 0x38;                       // P1.0 abilitata porta per ADC


	//Impostazioni del timerA con periodo di 20 ms e un duty Cycle minimo pari a 5 ms
	TACCR0 = periodo;                  	// PWM Periodo pari a 20 ms
	TACCTL1 = OUTMOD_7;                 // TACCR1 in modalit reset/set
	TACCR1 = min_duty;                  // TACCR1 Duty Cycle del PWM pari a 5 ms
	TACTL = TASSEL_2 + MC_1;            // SMCLK, upmode


	//Impostazioni del timerA1 con periodo di 20 ms e un duty Cycle minimo pari a 5 ms
	TA1CCR0 = periodo;                  // PWM Periodo pari a 20 ms
	TA1CCTL1 = OUTMOD_7;                // TA1CCR1 in modalit reset/set
	TA1CCTL2 = OUTMOD_7;                // TA1CCR2 in modalit reset/set
	TA1CCR1 = min_duty;                 // TA1CCR1 Duty Cycle del PWM pari a 5 ms
	TA1CCR2 = min_duty;                 // TA1CCR2 Duty Cycle del PWM pari a 5 ms
	TA1CTL = TASSEL_2 + MC_1;           // SMCLK, conteggio upmode


	/* Ciclo principale all'infinito. Legge i valori dei potenziometri per il controllo
	 * dei Futaba
	*/
	  while(1) {
	    __bis_SR_register(LPM0_bits + GIE);     // LPM0 + ABILITAZIONE IRQ
		
	    // Ogni volta che scade il tempo di WDT riparte da qui
	    ADC10CTL0 &= ~ENC;
	    while (ADC10CTL1 & BUSY);               // Attende che ADC10 sia attivo
		
		// (unsigned int) per rendere congrui i due termini
	    ADC10SA = (unsigned int)&adc_val[0];    // Dove sono trasferiti i dati
		
	    ADC10CTL0 |= ENC + ADC10SC;             // Icomincia il campionamento
		
	    adc_val[0] = adc_val[0]<<1;				// Moltiplico x 2
	    TACCR1 = 500 + adc_val[0];				// pari a Duty mim + val del pot
	    adc_val[1] = adc_val[1]<<1;				// Moltiplico x 2
	    TA1CCR1 = 500 + adc_val[1];				// pari a Duty mim + val del pot
	    adc_val[2] = adc_val[2]<<1;				// Moltiplico x 2
	    TA1CCR2 = 500 + adc_val[2];				// pari a Duty mim + val del pot

	  }
	}



	#pragma vector = WDT_VECTOR
	__interrupt void WDT_ISR(void){
	  __bic_SR_register_on_exit(LPM0_bits);     //  LPM0 esce da LPM0
	}


	








